@@ -1,9 +1,14 @@
 ################################################################################
 # TODO LIST
-# TODO: Issue error if (ref) not slimmed.
+# TODO: ...
 
 ################################################################################
 # CHANGE LOG
+# 18.07.2013: Check before overwrite object.
+# 15.07.2013: Added save GUI settings.
+# 11.06.2013: Added 'inherits=FALSE' to 'exists'.
+# 04.06.2013: Fixed bug in 'missingCol'.
+# 24.05.2013: Improved error message for missing columns.
 # 17.05.2013: listDataFrames() -> listObjects()
 # 09.05.2013: .result removed, added save as group.
 # 25.04.2013: First version.
@@ -16,18 +21,19 @@
 #' @details All data not matching the reference will be discarded. Useful for
 #' filtering stutters and artifacts from raw typing data.
 #' @param env environment in wich to search for data frames.
+#' @param savegui logical indicating if GUI settings should be saved in the environment.
 #' @param debug logical indicating printing debug information.
 #' 
 
-filterProfile_gui <- function(env=parent.frame(), debug=FALSE){
+filterProfile_gui <- function(env=parent.frame(), savegui=NULL, debug=FALSE){
   
   # Load dependencies.  
   require(gWidgets)
   options(guiToolkit="RGtk2")
   
-  gData <- NULL
-  gDataName <- NULL
-  gRef <- NULL
+  .gData <- NULL
+  .gDataName <- NULL
+  .gRef <- NULL
   
   if(debug){
     print(paste("IN:", match.call()[[1]]))
@@ -36,6 +42,11 @@ filterProfile_gui <- function(env=parent.frame(), debug=FALSE){
   
   w <- gwindow(title="Filter profile", visible=FALSE)
   
+  # Handler for saving GUI state.
+  addHandlerDestroy(w, handler = function (h, ...) {
+    .saveSettings()
+  })
+
   gv <- ggroup(horizontal=FALSE,
                spacing=8,
                use.scrollwindow=FALSE,
@@ -68,39 +79,59 @@ filterProfile_gui <- function(env=parent.frame(), debug=FALSE){
     
     val_obj <- svalue(dataset_drp)
     
-    if(exists(val_obj, envir=env)){
+    if(exists(val_obj, envir=env, inherits = FALSE)){
 
-      gData <<- get(val_obj, envir=env)
+      .gData <<- get(val_obj, envir=env)
       requiredCol <- c("Sample.Name", "Marker", "Allele", "Height")
+      slimmed <- sum(grepl("Allele",names(.gData), fixed=TRUE)) == 1
       
-      if(!all(requiredCol %in% colnames(gData))){
+      if(!all(requiredCol %in% colnames(.gData))){
   
-        gData <<- NULL
-        svalue(dataset_drp, index=TRUE) <- 1
-        svalue(g0_samples_lbl) <- " 0 samples"
-        svalue(f2_save_edt) <- ""
+        missingCol <- requiredCol[!requiredCol %in% colnames(.gData)]
         
-        message <- paste("The dataset is not typing data\n\n",
-                         "The following columns are required:\n",
-                         paste(requiredCol, collapse="\n"), sep="")
+        message <- paste("Additional columns required:\n",
+                         paste(missingCol, collapse="\n"), sep="")
         
         gmessage(message, title="Data error",
                  icon = "error",
                  parent = w) 
         
-      } else {
-  
-        gDataName <<- val_obj
+        # Reset components.
+        .gData <<- NULL
+        svalue(dataset_drp, index=TRUE) <- 1
+        svalue(g0_samples_lbl) <- " 0 samples"
+        svalue(f2_save_edt) <- ""
         
-        samples <- length(unique(gData$Sample.Name))
+      } else if (!slimmed) {
+        
+        message <- paste("The dataset is too fat!\n\n",
+                         "There can only be 1 'Allele' column\n",
+                         "Slim the dataset in the 'EDIT' tab", sep="")
+        
+        gmessage(message, title="message",
+                 icon = "error",
+                 parent = w) 
+        
+        # Reset components.
+        .gData <<- NULL
+        svalue(dataset_drp, index=TRUE) <- 1
+        svalue(g0_samples_lbl) <- " 0 samples"
+        svalue(f2_save_edt) <- ""
+        
+      } else {
+
+        # Load or change components.
+        .gDataName <<- val_obj
+        samples <- length(unique(.gData$Sample.Name))
         svalue(g0_samples_lbl) <- paste("", samples, "samples")
-        svalue(f2_save_edt) <- paste(gDataName, "_filter", sep="")
+        svalue(f2_save_edt) <- paste(.gDataName, "_filter", sep="")
         
       }
       
     } else {
       
-      gData <<- NULL
+      # Reset components.
+      .gData <<- NULL
       svalue(dataset_drp, index=TRUE) <- 1
       svalue(g0_samples_lbl) <- " 0 samples"
       svalue(f2_save_edt) <- ""
@@ -123,36 +154,56 @@ filterProfile_gui <- function(env=parent.frame(), debug=FALSE){
     
     val_obj <- svalue(refset_drp)
     
-    if(exists(val_obj, envir=env)){
+    if(exists(val_obj, envir=env, inherits = FALSE)){
       
-      gRef <<- get(val_obj, envir=env)
+      .gRef <<- get(val_obj, envir=env)
 
       requiredCol <- c("Sample.Name", "Marker", "Allele")
+      slimmed <- sum(grepl("Allele",names(.gRef), fixed=TRUE)) == 1
       
-      if(!all(requiredCol %in% colnames(gData))){
+      if(!all(requiredCol %in% colnames(.gData))){
         
-        gRef <<- NULL
-        svalue(refset_drp, index=TRUE) <- 1
-        svalue(g0_ref_lbl) <- " 0 references"
+        missingCol <- requiredCol[!requiredCol %in% colnames(.gRef)]
         
-        message <- paste("The dataset is not a reference dataset\n\n",
-                         "The following columns are required:\n",
-                         paste(requiredCol, collapse="\n"), sep="")
+        message <- paste("Additional columns required:\n",
+                         paste(missingCol, collapse="\n"), sep="")
         
         gmessage(message, title="Data error",
                  icon = "error",
                  parent = w) 
+
+        # Reset components.
+        .gRef <<- NULL
+        svalue(refset_drp, index=TRUE) <- 1
+        svalue(g0_ref_lbl) <- " 0 references"
+        
+      } else if (!slimmed) {
+        
+        message <- paste("The dataset is too fat!\n\n",
+                         "There can only be 1 'Allele' column\n",
+                         "Slim the dataset in the 'EDIT' tab", sep="")
+        
+        gmessage(message, title="message",
+                 icon = "error",
+                 parent = w) 
+        
+        # Reset components.
+        .gRef <<- NULL
+        svalue(refset_drp, index=TRUE) <- 1
+        svalue(g0_ref_lbl) <- " 0 references"
         
       } else {
-        
-        ref <- length(unique(gRef$Sample.Name))
+      
+        # Load or change components.
+        ref <- length(unique(.gRef$Sample.Name))
         svalue(g0_ref_lbl) <- paste("", ref, "references")
         
       }
       
     } else {
-      
-      gRef <<- NULL
+
+      # Reset components.
+      .gRef <<- NULL
       svalue(refset_drp, index=TRUE) <- 1
       svalue(g0_ref_lbl) <- " 0 references"
       
@@ -162,20 +213,24 @@ filterProfile_gui <- function(env=parent.frame(), debug=FALSE){
 
   # FRAME 1 ###################################################################
   
-  f1 <- gframe(text = "Filter settings",
+  f1 <- gframe(text = "Options",
                horizontal=FALSE,
                spacing = 5,
                container = gv) 
 
-  add_missing_loci_chk <- gcheckbox(text="Add missing loci",
-                                    checked = FALSE,
+  f1_savegui_chk <- gcheckbox(text="Save GUI settings",
+                              checked=FALSE,
+                              container=f1)
+  
+  f1_add_missing_loci_chk <- gcheckbox(text="Add missing loci",
+                                    checked = TRUE,
                                     container = f1)
   
-  keep_na_chk <- gcheckbox(text="Keep NA",
+  f1_keep_na_chk <- gcheckbox(text="Keep NA",
                            checked = FALSE,
                            container = f1)
   
-  ignore_case_chk <- gcheckbox(text="Ignore case",
+  f1_ignore_case_chk <- gcheckbox(text="Ignore case",
                            checked = TRUE,
                            container = f1)
 
@@ -199,25 +254,25 @@ filterProfile_gui <- function(env=parent.frame(), debug=FALSE){
   
   addHandlerChanged(filter_btn, handler = function(h, ...) {
     
-    val_add_missing_loci <- svalue(add_missing_loci_chk)
-    val_keep_na <- svalue(keep_na_chk)
-    val_ignore_case <- svalue(ignore_case_chk)
+    val_add_missing_loci <- svalue(f1_add_missing_loci_chk)
+    val_keep_na <- svalue(f1_keep_na_chk)
+    val_ignore_case <- svalue(f1_ignore_case_chk)
     val_name <- svalue(f2_save_edt)
     
-    if(!is.null(gData) & !is.null(gRef)){
+    if(!is.null(.gData) & !is.null(.gRef)){
       
       # Change button.
       svalue(filter_btn) <- "Processing..."
       enabled(filter_btn) <- FALSE
   
-      datanew <- filterProfile(data=gData,
-                               ref=gRef,
+      datanew <- filterProfile(data=.gData,
+                               ref=.gRef,
                                addMissingLoci=val_add_missing_loci,
                                keepNA=val_keep_na,
                                ignoreCase=val_ignore_case)
       
       # Save data.
-      assign(val_name, datanew, envir=env)
+      saveObject(name=val_name, object=datanew, parent=w, env=env)
       
       if(debug){
         print(datanew)
@@ -239,6 +294,88 @@ filterProfile_gui <- function(env=parent.frame(), debug=FALSE){
     
   } )
 
+  # INTERNAL FUNCTIONS ########################################################
+  
+  .loadSavedSettings <- function(){
+    
+    # First check status of save flag.
+    if(!is.null(savegui)){
+      svalue(f1_savegui_chk) <- savegui
+      enabled(f1_savegui_chk) <- FALSE
+      if(debug){
+        print("Save GUI status set!")
+      }  
+    } else {
+      # Load save flag.
+      if(exists(".filterProfile_gui_savegui", envir=env, inherits = FALSE)){
+        svalue(f1_savegui_chk) <- get(".filterProfile_gui_savegui", envir=env)
+      }
+      if(debug){
+        print("Save GUI status loaded!")
+      }  
+    }
+    if(debug){
+      print(svalue(f1_savegui_chk))
+    }  
+    
+    # Then load settings if true.
+    if(svalue(f1_savegui_chk)){
+      if(exists(".filterProfile_gui_add_loci", envir=env, inherits = FALSE)){
+        svalue(f1_add_missing_loci_chk) <- get(".filterProfile_gui_add_loci", envir=env)
+      }
+      if(exists(".filterProfile_gui_keep_na", envir=env, inherits = FALSE)){
+        svalue(f1_keep_na_chk) <- get(".filterProfile_gui_keep_na", envir=env)
+      }
+      if(exists(".filterProfile_gui_ignore_case", envir=env, inherits = FALSE)){
+        svalue(f1_ignore_case_chk) <- get(".filterProfile_gui_ignore_case", envir=env)
+      }
+      if(debug){
+        print("Saved settings loaded!")
+      }
+    }
+    
+  }
+  
+  .saveSettings <- function(){
+    
+    # Then save settings if true.
+    if(svalue(f1_savegui_chk)){
+      
+      assign(x=".filterProfile_gui_savegui", value=svalue(f1_savegui_chk), envir=env)
+      assign(x=".filterProfile_gui_add_loci", value=svalue(f1_add_missing_loci_chk), envir=env)
+      assign(x=".filterProfile_gui_keep_na", value=svalue(f1_keep_na_chk), envir=env)
+      assign(x=".filterProfile_gui_ignore_case", value=svalue(f1_ignore_case_chk), envir=env)
+      
+    } else { # or remove all saved values if false.
+      
+      if(exists(".filterProfile_gui_savegui", envir=env, inherits = FALSE)){
+        remove(".filterProfile_gui_savegui", envir = env)
+      }
+      if(exists(".filterProfile_gui_add_loci", envir=env, inherits = FALSE)){
+        remove(".filterProfile_gui_add_loci", envir = env)
+      }
+      if(exists(".filterProfile_gui_keep_na", envir=env, inherits = FALSE)){
+        remove(".filterProfile_gui_keep_na", envir = env)
+      }
+      if(exists(".filterProfile_gui_ignore_case", envir=env, inherits = FALSE)){
+        remove(".filterProfile_gui_ignore_case", envir = env)
+      }
+      
+      if(debug){
+        print("Settings cleared!")
+      }
+    }
+    
+    if(debug){
+      print("Settings saved!")
+    }
+    
+  }
+  
+  # END GUI ###################################################################
+  
+  # Load GUI settings.
+  .loadSavedSettings()
   
   # Show GUI.
   visible(w) <- TRUE
