@@ -5,6 +5,15 @@
 # NB! Can't handle Sample.Names as factors?
 ################################################################################
 # CHANGE LOG
+# 06.08.2013: Added rows and columns to info.
+# 18.07.2013: Check before overwrite object.
+# 16.07.2013: Added save GUI settings.
+# 11.06.2013: Added 'inherits=FALSE' to 'exists'.
+# 06.06.2013: Set initial table height to 200.
+# 04.06.2013: Fixed bug in 'missingCol'.
+# 24.05.2013: Fixed option 'replace missing values' "NA" -> NA.
+# 24.05.2013: Fixed option label for 'replace missing values'.
+# 24.05.2013: Improved error message for missing columns.
 # 17.05.2013: listDataFrames() -> listObjects()
 # 09.05.2013: .result removed, added save as group.
 # 27.04.2013: Add selection of dataset in gui. Removed parameter 'data'.
@@ -22,44 +31,55 @@
 #' user interface to it.
 #' 
 #' @param env environment in wich to search for data frames and save result.
+#' @param savegui logical indicating if GUI settings should be saved in the environment.
 #' @param debug logical indicating printing debug information.
 #' 
 #' @return data.frame with extracted result.
 
 
-trim_gui <- function(env=parent.frame(), debug=FALSE){
+trim_gui <- function(env=parent.frame(), savegui=NULL, debug=FALSE){
 
   # Load dependencies.  
   library(gWidgets)
   options(guiToolkit="RGtk2")
 
-  gData <- data.frame(Sample.Name="NA")
-  gDataName <- NULL
+  # Global variables.
+  .gData <- data.frame(Sample.Name="NA")
   
   if(debug){
     print(paste("IN:", match.call()[[1]]))
   }
-
+  
+  # Main window.
   w <- gwindow(title="Trim dataset", visible=FALSE)
   
+  # Handler for saving GUI state.
+  addHandlerDestroy(w, handler = function (h, ...) {
+    .saveSettings()
+  })
+
+  # Vertical main group.
   gv <- ggroup(horizontal=FALSE,
               spacing=5,
               use.scrollwindow=FALSE,
               container = w,
               expand=TRUE) 
 
+  # Vertical sub group.
   g0 <- ggroup(horizontal=FALSE,
                spacing=5,
                use.scrollwindow=FALSE,
                container = gv,
                expand=FALSE) 
 
+  # Horizontal sub group.
   g1 <- ggroup(horizontal=TRUE,
               spacing=5,
               use.scrollwindow=FALSE,
               container = gv,
               expand=TRUE) 
 
+  # Vertical sub group.
   g2 <- ggroup(horizontal=FALSE,
               spacing=5,
               use.scrollwindow=FALSE,
@@ -78,69 +98,82 @@ trim_gui <- function(env=parent.frame(), debug=FALSE){
                    spacing = 5,
                    container = g0) 
   
-  grid0 <- glayout(container = frame0, spacing = 1)
+  g0 <- glayout(container = frame0, spacing = 1)
   
-  grid0[1,1] <- glabel(text="Select dataset:", container=grid0)
+  g0[1,1] <- glabel(text="Select dataset:", container=g0)
   
-  grid0[1,2] <- dataset_drp <- gdroplist(items=c("<Select dataset>",
+  g0[1,2] <- dataset_drp <- gdroplist(items=c("<Select dataset>",
                                                  listObjects(env=env,
                                                              objClass="data.frame")),
                                          selected = 1,
                                          editable = FALSE,
-                                         container = grid0)
+                                         container = g0)
+  
+  g0[1,3] <- g0_samples_lbl <- glabel(text=" 0 samples,", container=g0)
+  g0[1,4] <- g0_columns_lbl <- glabel(text=" 0 columns,", container=g0)
+  g0[1,5] <- g0_rows_lbl <- glabel(text=" 0 rows", container=g0)
   
-  grid0[1,3] <- dataset_samples_lbl <- glabel(text=" 0 samples",
-                                              container=grid0)
   addHandlerChanged(dataset_drp, handler = function (h, ...) {
     
     val_obj <- svalue(dataset_drp)
     
-    if(exists(val_obj, envir=env)){
+    if(exists(val_obj, envir=env, inherits = FALSE)){
       
-      gData <<- get(val_obj, envir=env)
+      .gData <<- get(val_obj, envir=env)
 
       requiredCol <- c("Sample.Name")
       
-      if(!all(requiredCol %in% colnames(gData))){
+      if(!all(requiredCol %in% colnames(.gData))){
         
-        gData <<- data.frame(Sample.Name="NA")
-        svalue(sample_txt) <- ""
-        svalue(column_txt) <- ""
-        .refresh_samples_tbl()
-        .refresh_columns_tbl()
+        missingCol <- requiredCol[!requiredCol %in% colnames(.gData)]
         
-        svalue(dataset_samples_lbl) <- "0 samples"
-        svalue(f2_save_edt) <- ""
-        
-        message <- paste("The dataset is not sample data\n\n",
-                         "The following columns are required:\n",
-                         paste(requiredCol, collapse="\n"), sep="")
+        message <- paste("Additional columns required:\n",
+                         paste(missingCol, collapse="\n"), sep="")
         
         gmessage(message, title="Data error",
                  icon = "error",
                  parent = w) 
-        
-      } else {
-        
-        gDataName <<- val_obj
 
+        # Reset components.
+        .gData <<- data.frame(Sample.Name="NA")
+        svalue(sample_txt) <- ""
+        svalue(column_txt) <- ""
         .refresh_samples_tbl()
         .refresh_columns_tbl()
+        svalue(g0_samples_lbl) <- " 0 samples,"
+        svalue(g0_columns_lbl) <- " 0 columns,"
+        svalue(g0_rows_lbl) <- " 0 rows"
+        svalue(f2_save_edt) <- ""
         
-        samples <- length(unique(gData$Sample.Name))
-        svalue(dataset_samples_lbl) <- paste("", samples, "samples")
-        svalue(f2_save_edt) <- paste(gDataName, "_trim", sep="")
-        
+      } else {
+
+        # Load or change components.
+        .refresh_samples_tbl()
+        .refresh_columns_tbl()
+        # Info.
+        if("Sample.Name" %in% names(.gData)){
+          samples <- length(unique(.gData$Sample.Name))
+          svalue(g0_samples_lbl) <- paste(" ", samples, "samples,")
+        } else {
+          svalue(g0_samples_lbl) <- paste(" ", "<NA>", "samples,")
+        }
+        svalue(g0_columns_lbl) <- paste(" ", ncol(.gData), "columns,")
+        svalue(g0_rows_lbl) <- paste(" ", nrow(.gData), "rows")
+        # Result name.
+        svalue(f2_save_edt) <- paste(val_obj, "_trim", sep="")
       }
       
     } else {
       
-      gData <<- data.frame(Sample.Name="NA")
+      # Reset components.
+      .gData <<- data.frame(Sample.Name="NA")
       svalue(sample_txt) <- ""
       svalue(column_txt) <- ""
       .refresh_samples_tbl()
       .refresh_columns_tbl()
-      svalue(dataset_samples_lbl) <- " 0 samples"
+      svalue(g0_samples_lbl) <- paste(" ", "<NA>", "samples,")
+      svalue(g0_columns_lbl) <- paste(" ", "<NA>", "columns,")
+      svalue(g0_rows_lbl) <- paste(" ", "<NA>", "rows")
       svalue(f2_save_edt) <- ""
       
     }
@@ -168,10 +201,13 @@ trim_gui <- function(env=parent.frame(), debug=FALSE){
                       width = 40,
                       container=sample_f)
 
-  sample_tbl <- gtable(items=data.frame(Sample.Names=unique(gData$Sample.Name),
+  sample_tbl <- gtable(items=data.frame(Sample.Names=unique(.gData$Sample.Name),
                                         stringsAsFactors=FALSE),
                        container=sample_f,
                        expand=TRUE)
+  
+  # Set initial size (only height is important here).
+  size(sample_tbl) <- c(100,200)
 
   addDropTarget(sample_txt, handler=function(h,...) {
     # Get values.
@@ -217,7 +253,7 @@ trim_gui <- function(env=parent.frame(), debug=FALSE){
                       container=column_f)
   
   
-  column_tbl <- gtable(items=names(gData), 
+  column_tbl <- gtable(items=names(.gData), 
                        container=column_f,
                        expand=TRUE)
 
@@ -252,6 +288,10 @@ trim_gui <- function(env=parent.frame(), debug=FALSE){
                      container=g1, 
                      expand=TRUE)
   
+  savegui_chk <- gcheckbox(text="Save GUI settings",
+                              checked=FALSE,
+                              container=option_f)
+
   empty_chk <- gcheckbox(text="Remove empty columns",
                          checked=TRUE,
                          container=option_f)
@@ -268,8 +308,9 @@ trim_gui <- function(env=parent.frame(), debug=FALSE){
                         checked=TRUE, 
                         container=option_f)
   
+  glabel(text="Replace missing values with:",
+                  container=option_f)
   na_txt <- gedit(text="NA",
-                  label="Replace missing values with:",
                   container=option_f)
 
   # FRAME 2 ###################################################################
@@ -314,10 +355,15 @@ trim_gui <- function(env=parent.frame(), debug=FALSE){
       na_val <- svalue(na_chk)
       empty_val <- svalue(empty_chk)
       na_txt_val <- svalue(na_txt)
+
+      # NA can't be string.
+      if(na_txt_val == "NA"){
+        na_txt_val <- NA
+      }
       
       if(debug){
-        print("gData")
-        print(names(gData))
+        print(".gData")
+        print(names(.gData))
         print("column_val")
         print(column_val)
         print("word_val")
@@ -340,14 +386,13 @@ trim_gui <- function(env=parent.frame(), debug=FALSE){
       svalue(trim_btn) <- "Processing..."
       enabled(trim_btn) <- FALSE
       
-      datanew <- trim(data=gData, samples=sample_val, columns=column_val, 
+      datanew <- trim(data=.gData, samples=sample_val, columns=column_val, 
                    word=word_val, ignoreCase=case_val, invertS=sample_opt_val, invertC=column_opt_val,
                    rmNaCol=na_val, rmEmptyCol=empty_val, missing=na_txt_val)
   
-  
       # Save data.
-      assign(val_name, datanew, envir=env)
-  
+      saveObject(name=val_name, object=datanew, parent=w, env=env)
+      
       # Close GUI.
       dispose(w)
     
@@ -360,9 +405,6 @@ trim_gui <- function(env=parent.frame(), debug=FALSE){
     
   } )
   
-  # Show GUI.
-  visible(w) <- TRUE
-  
   .refresh_samples_tbl <- function(){
     
     if(debug){
@@ -373,7 +415,7 @@ trim_gui <- function(env=parent.frame(), debug=FALSE){
     delete(sample_f, sample_tbl)
     
     # ...creating a new table.
-    sample_tbl <<- gtable(items=data.frame(Sample.Names=unique(gData$Sample.Name),
+    sample_tbl <<- gtable(items=data.frame(Sample.Names=unique(.gData$Sample.Name),
                                            stringsAsFactors=FALSE),
                        container=sample_f,
                        expand=TRUE)
@@ -405,6 +447,8 @@ trim_gui <- function(env=parent.frame(), debug=FALSE){
     
   }
   
+  # INTERNAL FUNCTIONS ########################################################
+  
   .refresh_columns_tbl <- function(){
     
     if(debug){
@@ -415,7 +459,7 @@ trim_gui <- function(env=parent.frame(), debug=FALSE){
     delete(column_f, column_tbl)
     
     # ...creating a new table.
-    column_tbl <<- gtable(items=names(gData), 
+    column_tbl <<- gtable(items=names(.gData), 
                          container=column_f,
                          expand=TRUE)
     
@@ -443,5 +487,118 @@ trim_gui <- function(env=parent.frame(), debug=FALSE){
     } )
     
   }
+  
+  .loadSavedSettings <- function(){
     
-} # End of GUI
+    # First check status of save flag.
+    if(!is.null(savegui)){
+      svalue(savegui_chk) <- savegui
+      enabled(savegui_chk) <- FALSE
+      if(debug){
+        print("Save GUI status set!")
+      }  
+    } else {
+      # Load save flag.
+      if(exists(".trim_gui_savegui", envir=env, inherits = FALSE)){
+        svalue(savegui_chk) <- get(".trim_gui_savegui", envir=env)
+      }
+      if(debug){
+        print("Save GUI status loaded!")
+      }  
+    }
+    if(debug){
+      print(svalue(savegui_chk))
+    }  
+        
+    # Then load settings if true.
+    if(svalue(savegui_chk)){
+      if(exists(".trim_gui_sample_option", envir=env, inherits = FALSE)){
+        svalue(sample_opt) <- get(".trim_gui_sample_option", envir=env)
+      }
+      if(exists(".trim_gui_column_option", envir=env, inherits = FALSE)){
+        svalue(column_opt) <- get(".trim_gui_column_option", envir=env)
+      }
+      if(exists(".trim_gui_remove_empty", envir=env, inherits = FALSE)){
+        svalue(empty_chk) <- get(".trim_gui_remove_empty", envir=env)
+      }
+      if(exists(".trim_gui_remove_na", envir=env, inherits = FALSE)){
+        svalue(na_chk) <- get(".trim_gui_remove_na", envir=env)
+      }
+      if(exists(".trim_gui_add_word", envir=env, inherits = FALSE)){
+        svalue(word_chk) <- get(".trim_gui_add_word", envir=env)
+      }
+      if(exists(".trim_gui_ignore_case", envir=env, inherits = FALSE)){
+        svalue(case_chk) <- get(".trim_gui_ignore_case", envir=env)
+      }
+      if(exists(".trim_gui_replace_na", envir=env, inherits = FALSE)){
+        svalue(na_txt) <- get(".trim_gui_replace_na", envir=env)
+      }
+      
+      if(debug){
+        print("Saved settings loaded!")
+      }
+    }
+    
+  }
+  
+  .saveSettings <- function(){
+    
+    # Then save settings if true.
+    if(svalue(savegui_chk)){
+      
+      assign(x=".trim_gui_savegui", value=svalue(savegui_chk), envir=env)
+      assign(x=".trim_gui_sample_option", value=svalue(sample_opt), envir=env)
+      assign(x=".trim_gui_column_option", value=svalue(column_opt), envir=env)
+      assign(x=".trim_gui_remove_empty", value=svalue(empty_chk), envir=env)
+      assign(x=".trim_gui_remove_na", value=svalue(na_chk), envir=env)
+      assign(x=".trim_gui_add_word", value=svalue(word_chk), envir=env)
+      assign(x=".trim_gui_ignore_case", value=svalue(case_chk), envir=env)
+      assign(x=".trim_gui_replace_na", value=svalue(na_txt), envir=env)
+      
+    } else { # or remove all saved values if false.
+      
+      if(exists(".trim_gui_savegui", envir=env, inherits = FALSE)){
+        remove(".trim_gui_savegui", envir = env)
+      }
+      if(exists(".trim_gui_sample_option", envir=env, inherits = FALSE)){
+        remove(".trim_gui_sample_option", envir = env)
+      }
+      if(exists(".trim_gui_column_option", envir=env, inherits = FALSE)){
+        remove(".trim_gui_column_option", envir = env)
+      }
+      if(exists(".trim_gui_remove_empty", envir=env, inherits = FALSE)){
+        remove(".trim_gui_remove_empty", envir = env)
+      }
+      if(exists(".trim_gui_remove_na", envir=env, inherits = FALSE)){
+        remove(".trim_gui_remove_na", envir = env)
+      }
+      if(exists(".trim_gui_add_word", envir=env, inherits = FALSE)){
+        remove(".trim_gui_add_word", envir = env)
+      }
+      if(exists(".trim_gui_ignore_case", envir=env, inherits = FALSE)){
+        remove(".trim_gui_ignore_case", envir = env)
+      }
+      if(exists(".trim_gui_replace_na", envir=env, inherits = FALSE)){
+        remove(".trim_gui_replace_na", envir = env)
+      }
+      
+      if(debug){
+        print("Settings cleared!")
+      }
+    }
+    
+    if(debug){
+      print("Settings saved!")
+    }
+    
+  }
+  
+  # END GUI ###################################################################
+  
+  # Load GUI settings.
+  .loadSavedSettings()
+  
+  # Show GUI.
+  visible(w) <- TRUE
+  
+}
